@@ -12,13 +12,11 @@ import pyparsing as pp
 
 from .match_environ import MatchContext
 
-
 IMPLICIT_BIN_OP_AND = 'AND'
 IMPLICIT_BIN_OP_OR = 'OR'
 TERM_MATCH_OP_EQUAL = 'EQUAL'
 TERM_MATCH_OP_CONTAIN = 'CONTAIN'
 
-
 pp.ParserElement.enablePackrat()
 
 
@@ -42,7 +40,7 @@ RQuotedString = collections.namedtuple('RQuotedString', ['value', 'options'])
 def flat_nested_object(o, pathes=None, depth=0, max_depth=10):
     if depth >= max_depth:
         return
-    if pathes == None:
+    if pathes is None:
         pathes = []
     if isinstance(o, list):
         for idx, item in enumerate(o):
@@ -71,13 +69,16 @@ class BaseMatcher:
         """
         raise TypeError
 
+    def eval_one(self, input_value, context):
+        raise NotImplemented
+
     def eval(self, input_value, context):
         # 시작하기 전에 한번 검사해서 flat_nested_object 에 의한 generator 생성을 차단
         if isinstance(input_value, numbers.Number) or isinstance(input_value, string_types):
             return self.eval_one(input_value, context)
 
         for name, value in flat_nested_object(input_value):
-            if isinstance(input_value , list) and len(input_value) and isinstance(input_value[0] , dict):
+            if isinstance(input_value, list) and len(input_value) and isinstance(input_value[0], dict):
                 break
             matched, matched_value = self.eval_one(value, context)
             if matched:
@@ -97,6 +98,18 @@ class TextMatcher(BaseMatcher):
     def __repr__(self):
         return 'TextMatcher:{}'.format(self.value)
 
+    def count(self, input_value, context):
+        if not isinstance(input_value, string_types):
+            input_value = str(input_value)
+
+        # keyword set 설정이 있는 경우 반영한다.
+        if context.has_keyword_set(self.value):
+            count, matched_value = context.count_keyword_set(self.value, input_value)
+            return count, matched_value
+        else:
+            count = input_value.count(self.value)
+            return count, self.value
+
     def eval_one(self, input_value, context):
         # input_value 가 숫자형태인 경우 Int/Float 변환을 통한 매치를 우선 시도한다.
         if not self.quoted and isinstance(input_value, numbers.Number):
@@ -151,6 +164,27 @@ class RegexpMatcher(BaseMatcher):
     def __repr__(self):
         return 'RegexpMatcher:{}'.format(self.value)
 
+    def count(self, input_value, context):
+        if not isinstance(input_value, string_types):
+            input_value = str(input_value)
+
+        # keyword set 설정이 있는 경우 반영한다.
+        pattern = self.pattern
+        if context.has_keyword_set(self.value):
+            pattern = context.expand_regexp(self.value)
+
+        last_matched = ''
+        count = 0
+        start = 0
+        while True:
+            m = pattern.search(input_value[start:])
+            if not m:
+                break
+            last_matched = m.group()
+            start += m.start() + 1
+            count += 1
+        return count, last_matched
+
     def eval_one(self, input_value, context):
         if not isinstance(input_value, string_types):
             input_value = str(input_value)
@@ -194,6 +228,95 @@ class MultipleTextMatcher(BaseMatcher):
         return False, None
 
 
+class CountingMatcher(BaseMatcher):
+    def __init__(self, matcher, op, condition_value):
+        self.matcher = matcher
+        self.op = op
+        self.condition_value = int(condition_value.value)
+
+    def __repr__(self):
+        return f'CountingMatcher({self.matcher},{self.op},{self.condition_value})'
+
+    def count(self, value, context):
+        count, matched_value = self.matcher.count(value, context)
+        return count, matched_value
+
+    def eval_one(self, input_value, context):
+        return self.count(input_value, context)
+
+    def eval(self, input_value, context):
+        count = 0
+        last_matched = ''
+
+        # 시작하기 전에 한번 검사해서 flat_nested_object 에 의한 generator 생성을 차단
+        if isinstance(input_value, numbers.Number) or isinstance(input_value, string_types):
+            count, last_matched = self.eval_one(input_value, context)
+        else:
+            for name, value in flat_nested_object(input_value):
+                if isinstance(input_value, list) and len(input_value) and isinstance(input_value[0], dict):
+                    break
+                e_count, e_last_matched = self.eval_one(value, context)
+                if e_count > 0:
+                    count += e_count
+                    last_matched = e_last_matched
+
+        if self.op == '<=':
+            return count <= self.condition_value, last_matched
+        elif self.op == '<':
+            return count < self.condition_value, last_matched
+        elif self.op == '>=':
+            return count >= self.condition_value, last_matched
+        elif self.op == '>':
+            return count > self.condition_value, last_matched
+        elif self.op == '=':
+            pass
+        return count == self.condition_value, last_matched
+
+
+class CodeMatcher:
+    def __init__(self, expression):
+        self.expression = expression
+        self.compiled = __builtins__['compile'](expression, '_expression_matcher', 'eval')
+        parsed = ast.parse(expression)
+        self.variable_names = [node.id for node in ast.walk(parsed) if isinstance(node, ast.Name)]
+
+    def __repr__(self):
+        return 'CodeMatcher({})'.format(self.expression)
+
+    def create_local(self, input_value, context):
+        local = {}
+        functions = context.environ.get_functions()
+
+        local.update(functions)
+        local['this'] = input_value
+        return local
+
+    def is_no_args_function(self, local):
+        first = self.variable_names[0] if len(self.variable_names) == 1 else None
+        if first and callable(local.get(first)):
+            return True
+        else:
+            return False
+
+    def call_single_function(self, local, this):
+        first = self.variable_names[0]
+        return local[first](this)
+
+    def eval(self, input_value, context):
+        this = input_value
+        local = self.create_local(input_value, context)
+        try:
+            if self.is_no_args_function(local):
+                ret = self.call_single_function(local, this)
+            else:
+                ret = eval(self.compiled, {}, local)
+        except (AttributeError, TypeError) as e:
+            return False, None
+        except Exception as e:
+            raise e
+        return ret, ret
+
+
 class Operator(BaseMatcher):
     def __init__(self, op, value):
         self.op = op
@@ -248,7 +371,7 @@ class RangeMatcher(BaseMatcher):
             start = float(start)
             stop = float(stop)
             self.is_float = True
-        except:
+        except Exception:
             self.is_float = False
         self.start = start
         self.stop = stop
@@ -289,7 +412,7 @@ class TermMatcher:
 
     def eval(self, context):
         input_value = context.get(self.field_name)
-        
+
         if input_value is None:
             return False, None
 
@@ -329,11 +452,17 @@ class ExpressionMatcher:
     def __repr__(self):
         return 'ExpressionMatcher({})'.format(self.expression)
 
+    def get_local(self, context):
+        functions = context.environ.get_functions()
+        local = dict(context.get_dict())
+        local.update(functions)
+        return DictOrObject(local)
+
     def eval(self, context):
         try:
-            local = DictOrObject(context.get_dict())
+            local = self.get_local(context)
             ret = eval(self.compiled, {}, local)
-        except (AttributeError, TypeError):
+        except (AttributeError, TypeError) as e:
             return False, None
         except Exception as e:
             raise e
@@ -452,12 +581,13 @@ def get_parser(implicit_bin_op=IMPLICIT_BIN_OP_AND, term_match_op=TERM_MATCH_OP_
     AND_, OR_, NOT_, TO_ = map(pp.CaselessKeyword, 'AND OR NOT TO'.split())
     LTE, LT, GTE, GT, EQ  = map(pp.Literal, ['<=', '<', '>=', '>', '='])
     RAW_REGEXP_DELIMETER = pp.Literal('/~/')
-
-    keyword = AND_ | OR_ | NOT_ | TO_
+    COUNT_NAME = 'COUNT'
+    START_OF_CODE = '!'
 
     expression = pp.Forward()
 
     valid_keyword = pp.Regex(r'[a-zA-Z_*@][a-zA-Z0-9_*@.\[\]]*')
+    valid_code = pp.Regex(r'([^\s]+)').setParseAction(lambda t: ValidText(t[0]))
     valid_text = pp.Regex(r'([^\s\)]+)').setParseAction(lambda t: ValidText(t[0]))
     quoted_string = pp.QuotedString('"').setParseAction(lambda t: QuotedString(t[0]))
     rquoted_string = (pp.QuotedString('/', escChar='\\', escQuote='\\/', unquoteResults=False)
@@ -483,8 +613,17 @@ def get_parser(implicit_bin_op=IMPLICIT_BIN_OP_AND, term_match_op=TERM_MATCH_OP_
     field_multiple_value = (LPAR + pp.OneOrMore(multiple_field_text_value) + RPAR)('multiple_field_value') \
         .setParseAction(lambda t: MultipleTextMatcher(t, term_match_op))
 
+    field_count_value = (COUNT_NAME + LPAR + field_text_value + RPAR + (LTE | LT | GTE | GT | EQ) + (valid_text)) \
+        .setParseAction(lambda t: CountingMatcher(t[1], t[2], t[3]))
+
+    field_code_string = (quoted_string | valid_code)('code_field_value')
+    field_code_value = (START_OF_CODE + field_code_string) \
+        .setParseAction(lambda t: CodeMatcher(t[1].value))
+
     field_name = valid_keyword
-    field_value = (field_multiple_value | field_operate_value | field_range_value | field_text_value)
+    field_value = (field_code_value | field_count_value |
+                   field_multiple_value | field_operate_value |
+                   field_range_value | field_text_value)
     field_term = pp.Group(field_name('field_name') + COLON + field_value) \
         .setParseAction(lambda t: build_term_matcher(t[0][0], t[0][2]))
 
@@ -573,7 +712,6 @@ IMPLICIT_AND = 1 << 1
 TERM_MATCH_EQUAL = 1 << 2
 TERM_MATCH_CONTAIN = 1 << 3
 
-
 default_term_match_op = TERM_MATCH_OP_EQUAL
 
 
